# {{ introduction_title }}



## 简要描述

{{ introduction_summary }}



## 请求URL

- `{{ request_url }}`



## 请求方式

{% if view_class_type == "CompleteModelView" %}
- GET查, POST增, PUT改, DELETE删
{% elif view_class_type == "BaseFullTextSearchView" %}
- GET
{% elif view_class_type == "BaseListView" %}
- GET
{% elif view_class_type == "AdvancedSearchView" %}
- POST
{% else %}
- 其它 {{view_class_type}}
{% endif %}



## 请求参数


- 基本参数

| 类型 | 参数名            | 说明                                                    | 必填 |
| ---- | ----------------- | ------------------------------------------------------- |--- |
| int  | p                 | 页码                                                    | 否 |
| int  | page_size         | 每页数量                                                | 否 |
| list | order_type_ls     | 排序字段名, 如`-id`为倒序排列                             | 否 |

- 过滤字段
{% if has_filter_fields %}
> 默认精确检索. 模糊检索示例: `{{ '{{' }}字段名{{ '}}' }}__icontains=xxx`, 如`name__icontains=张`
{% endif %}
{{ filter_fields__doc }}



## 请求示例


```
{{ introduction_url }}
```



## 返回示例


```

{{ example_data }}

```



## 返回参数说明


- 基本参数

| 类型 | 参数名 | 说明        |
| ---- | ------ | ----------- |
| int  | {{ response_render_format.status }} | 200,201成功 |
| int  | {{ response_render_format.msg }}    | 信息        |
| dict | {{ response_render_format.result }} | 返回结果    |

- `result`中的`page_dc`页码信息

| 类型 | 参数名      | 说明     |
| ---- | ----------- | -------- |
| int  | count_items | 总条数   |
| int  | total_pages | 总页数   |
| int  | page_size   | 每页条数 |
| int  | p           | 当前页   |

- 模型字段

{{ parameters_explain }}



## 备注

- {{ context_request_type }}
- [API文档格式说明](https://wiki-bddjango.readthedocs.io/zh/latest/FormatDescription/ "点击跳转")


